package my.test.sakila.view;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.Stateful;
import javax.enterprise.context.Conversation;
import javax.enterprise.context.ConversationScoped;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.inject.Inject;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import my.test.sakila.model.FilmCategory;
import my.test.sakila.model.Category;
import my.test.sakila.model.Film;

/**
 * Backing bean for FilmCategory entities.
 * <p>
 * This class provides CRUD functionality for all FilmCategory entities. It focuses
 * purely on Java EE 6 standards (e.g. <tt>&#64;ConversationScoped</tt> for
 * state management, <tt>PersistenceContext</tt> for persistence,
 * <tt>CriteriaBuilder</tt> for searches) rather than introducing a CRUD framework or
 * custom base class.
 */

@Named
@Stateful
@ConversationScoped
public class FilmCategoryBean implements Serializable {

	private static final long serialVersionUID = 1L;

	/*
	 * Support creating and retrieving FilmCategory entities
	 */

	private Long id;

	public Long getId() {
		return this.id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	private FilmCategory filmCategory;

	public FilmCategory getFilmCategory() {
		return this.filmCategory;
	}

	@Inject
	private Conversation conversation;

	@PersistenceContext(type = PersistenceContextType.EXTENDED)
	private EntityManager entityManager;

	public String create() {

		this.conversation.begin();
		return "create?faces-redirect=true";
	}
	
	public void retrieve() {

		if (FacesContext.getCurrentInstance().isPostback()) {
			return;
		}

		if (this.conversation.isTransient()) {
			this.conversation.begin();
		}

		if (this.id == null) {
			this.filmCategory = this.search;
		} else {
			this.filmCategory = this.entityManager.find(FilmCategory.class, getId());
		}
	}

	/*
	 * Support updating and deleting FilmCategory entities
	 */

	public String update() {
		this.conversation.end();
		
		try {
			if (this.id == null) {
				this.entityManager.persist(this.filmCategory);
				return "search?faces-redirect=true";			
			} else {
				this.entityManager.merge(this.filmCategory);
				return "view?faces-redirect=true&id=" + this.filmCategory.getId();
			}
		} catch( Exception e ) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage( e.getMessage() ));
			return null;
		}
	}

	public String delete() {
		this.conversation.end();

		try {
			this.entityManager.remove(this.entityManager.find(FilmCategory.class,
					getId()));
			this.entityManager.flush();
			return "search?faces-redirect=true";
		} catch( Exception e ) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage( e.getMessage() ));
			return null;
		}
	}

	/*
	 * Support searching FilmCategory entities with pagination
	 */

	private int page;
	private long count;
	private List<FilmCategory> pageItems;
	
	private FilmCategory search = new FilmCategory();

	public int getPage() {
		return this.page;
	}

	public void setPage(int page) {
		this.page = page;
	}

	public int getPageSize() {
		return 10;
	}

	public FilmCategory getSearch() {
		return this.search;
	}

	public void setSearch(FilmCategory search) {
		this.search = search;
	}

	public void search() {
		this.page = 0;
	}

	public void paginate() {

		CriteriaBuilder builder = this.entityManager.getCriteriaBuilder();

		// Populate this.count

		CriteriaQuery<Long> countCriteria = builder.createQuery(Long.class);
		Root<FilmCategory> root = countCriteria.from(FilmCategory.class);
		countCriteria = countCriteria.select(builder.count(root)).where(
				getSearchPredicates(root));
		this.count = this.entityManager.createQuery(countCriteria)
				.getSingleResult();

		// Populate this.pageItems

		CriteriaQuery<FilmCategory> criteria = builder.createQuery(FilmCategory.class);
		root = criteria.from(FilmCategory.class);
		TypedQuery<FilmCategory> query = this.entityManager.createQuery(criteria
				.select(root).where(getSearchPredicates(root)));
		query.setFirstResult(this.page * getPageSize()).setMaxResults(
				getPageSize());
		this.pageItems = query.getResultList();
	}

	private Predicate[] getSearchPredicates(Root<FilmCategory> root) {

		CriteriaBuilder builder = this.entityManager.getCriteriaBuilder();
		List<Predicate> predicatesList = new ArrayList<Predicate>();

		Film film = this.search.getFilm();
		if (film != null && film.getFilmId() != null) {
			predicatesList.add(builder.equal(root.get("film"), film));
		}
		Category category = this.search.getCategory();
		if (category != null && category.getCategoryId() != null) {
			predicatesList.add(builder.equal(root.get("category"), category));
		}

		return predicatesList.toArray(new Predicate[predicatesList.size()]);
	}

	public List<FilmCategory> getPageItems() {
		return this.pageItems;
	}

	public long getCount() {
		return this.count;
	}

	/*
	 * Support listing and POSTing back FilmCategory entities (e.g. from inside an
	 * HtmlSelectOneMenu)
	 */

	public List<FilmCategory> getAll() {

		CriteriaQuery<FilmCategory> criteria = this.entityManager
				.getCriteriaBuilder().createQuery(FilmCategory.class);
		return this.entityManager.createQuery(
				criteria.select(criteria.from(FilmCategory.class))).getResultList();
	}

	public Converter getConverter() {

		return new Converter() {

			@Override
			public Object getAsObject(FacesContext context,
					UIComponent component, String value) {

				return FilmCategoryBean.this.entityManager.find(FilmCategory.class,
						Long.valueOf(value));
			}

			@Override
			public String getAsString(FacesContext context,
					UIComponent component, Object value) {

				if (value == null) {
					return "";
				}

				return String.valueOf(((FilmCategory) value).getId());
			}
		};
	}
	
	/*
	 * Support adding children to bidirectional, one-to-many tables
	 */
	 
	private FilmCategory add = new FilmCategory();

	public FilmCategory getAdd() {
		return this.add;
	}

	public FilmCategory getAdded() {
		FilmCategory added = this.add;
		this.add = new FilmCategory();
		return added;
	}
}